/* *****************************************************************************
 * libg1m/format/cas.h -- the older CAS file format description.
 * Copyright (C) 2017 Thomas "Cakeisalie5" Touhey <thomas@touhey.fr>
 *
 * This file is part of libg1m.
 * libg1m is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3.0 of the License,
 * or (at your option) any later version.
 *
 * libg1m is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with libg1m; if not, see <http://www.gnu.org/licenses/>.
 * ************************************************************************** */
#ifndef LIBG1M_FORMAT_CAS_H
# define LIBG1M_FORMAT_CAS_H
# include <stdint.h>
# pragma pack(1)

/* This is the first CASIOLINK format. It is more or less a raw dump of the
 * communications protocol the 9700 uses. It is basically an MCS file.
 *
 * A CASIOLINK-encoded file is made of one header and one or more parts.
 * Each element is preceded by a 0x3A, and is a packet in the communications
 * protocol interpretation. */
/* ************************************************************************** */
/*  Header                                                                    */
/* ************************************************************************** */
/* The header is the following: */

struct cas_header {
	uint8_t data[2];
	uint8_t misc[5]; /* type-specific subheader (miscallaneous data) */
	uint8_t filename[12]; /* editor filename. */
	uint8_t password[12]; /* editor password. */
	uint8_t _reserved[7]; /* 0xFFs */
	uint8_t checksum;
};

/* For a program, the specific bytes are the following: */

# define casiolink_program_uk0     0x01 /* <unknown> */
# define casiolink_program_stats   0x02 /* store stats data */
# define casiolink_program_matrix  0x04 /* matrix mode */
# define casiolink_program_uk1     0x08 /* <unknown */
# define casiolink_program_stddev  0x10 /* standard deviation mode */
# define casiolink_program_linreg  0x20 /* linear regression mode */
# define casiolink_program_basen   0x40 /* base-n mode */
# define casiolink_program_drawst  0x80 /* draw stats graph */

struct cas_spe_program {
	uint8_t _unknown;
	uint16_t length;
	uint8_t flags;
	uint8_t _unknown2;
};

/* The specific bytes are the same for a set of programs, but `flags` is
 * not used, and the length is the size of all of the programs put together.
 *
 * Now, here are the specific bytes for a screenshot: */

struct cas_spe_screenshot {
	uint8_t height;
	uint8_t width;
	uint8_t _unused[3];
};

/* The specific bytes for a number start with either "RA" or "CA", 'R' or 'C'
 * meaning the number is complex or not. */
/* ************************************************************************** */
/*  Content (parts)                                                           */
/* ************************************************************************** */
/* TODO */

# pragma pack()
#endif /* LIBG1M_FORMAT_CAS_H */
